@@ -41,10 +41,8 @@ class Arbiter(object):
 
     # I love dynamic languages
     SIG_QUEUE = []
-    SIGNALS = map(
-        lambda x: getattr(signal, "SIG%s" % x),
-        "HUP QUIT INT TERM TTIN TTOU USR1 USR2 WINCH".split()
-    )
+    SIGNALS = [getattr(signal, "SIG%s" % x) \
+            for x in "HUP QUIT INT TERM TTIN TTOU USR1 USR2 WINCH".split()]
     SIG_NAMES = dict(
         (getattr(signal, name), name[3:].lower()) for name in dir(signal)
         if name[:3] == "SIG" and name[3] != "_"
@@ -87,7 +85,8 @@ class Arbiter(object):
         self.log = self.cfg.logger_class(app.cfg)
 
         # reopen files
-        self.log.reopen_files()
+        if 'GUNICORN_FD' in os.environ:
+            self.log.reopen_files()
 
         self.address = self.cfg.address
         self.num_workers = self.cfg.workers
@@ -98,7 +97,10 @@ class Arbiter(object):
 
         if self.cfg.debug:
             self.log.debug("Current configuration:")
-            for config, value in sorted(self.cfg.settings.iteritems()):
+
+
+            for config, value in sorted(self.cfg.settings.items(),
+                    key=lambda setting: setting[1]):
                 self.log.debug("  %s: %s", config, value.value)
 
         if self.cfg.preload_app:
@@ -112,15 +114,15 @@ class Arbiter(object):
         Initialize the arbiter. Start listening and set pidfile if needed.
         """
         self.log.info("Starting gunicorn %s", __version__)
-        self.cfg.on_starting(self)
         self.pid = os.getpid()
+        if self.cfg.pidfile is not None:
+            self.pidfile = Pidfile(self.cfg.pidfile)
+            self.pidfile.create(self.pid)
+        self.cfg.on_starting(self)
         self.init_signals()
         if not self.LISTENER:
             self.LISTENER = create_socket(self.cfg, self.log)
 
-        if self.cfg.pidfile is not None:
-            self.pidfile = Pidfile(self.cfg.pidfile)
-            self.pidfile.create(self.pid)
         self.log.debug("Arbiter booted")
         self.log.info("Listening at: %s (%s)", self.LISTENER,
             self.pid)
@@ -134,13 +136,20 @@ class Arbiter(object):
         Initialize master signal handling. Most of the signals
         are queued. Child signals only wake up the master.
         """
+        # close old PIPE
         if self.PIPE:
-            map(os.close, self.PIPE)
+            [os.close(p) for p in self.PIPE]
+
+        # initialize the pipe
         self.PIPE = pair = os.pipe()
-        map(util.set_non_blocking, pair)
-        map(util.close_on_exec, pair)
+        for p in pair:
+            util.set_non_blocking(p)
+            util.close_on_exec(p)
+
         self.log.close_on_exec()
-        map(lambda s: signal.signal(s, self.signal), self.SIGNALS)
+
+        # intialiatze all signals
+        [signal.signal(s, self.signal) for s in self.SIGNALS]
         signal.signal(signal.SIGCHLD, self.handle_chld)
 
     def signal(self, sig, frame):
@@ -180,7 +189,7 @@ class Arbiter(object):
                 self.halt()
             except KeyboardInterrupt:
                 self.halt()
-            except HaltServer, inst:
+            except HaltServer as inst:
                 self.halt(reason=inst.reason, exit_status=inst.exit_status)
             except SystemExit:
                 raise
@@ -269,8 +278,8 @@ class Arbiter(object):
         Wake up the arbiter by writing to the PIPE
         """
         try:
-            os.write(self.PIPE[1], '.')
-        except IOError, e:
+            os.write(self.PIPE[1], b'.')
+        except IOError as e:
             if e.errno not in [errno.EAGAIN, errno.EINTR]:
                 raise
 
@@ -295,10 +304,10 @@ class Arbiter(object):
                 return
             while os.read(self.PIPE[0], 1):
                 pass
-        except select.error, e:
-            if e[0] not in [errno.EAGAIN, errno.EINTR]:
+        except select.error as e:
+            if e.args[0] not in [errno.EAGAIN, errno.EINTR]:
                 raise
-        except OSError, e:
+        except OSError as e:
             if e.errno not in [errno.EAGAIN, errno.EINTR]:
                 raise
         except KeyboardInterrupt:
@@ -353,6 +362,9 @@ class Arbiter(object):
         self.app.reload()
         self.setup(self.app)
 
+        # reopen log files
+        self.log.reopen_files()
+
         # do we need to change listener ?
         if old_address != self.cfg.address:
             self.LISTENER.close()
@@ -385,6 +397,8 @@ class Arbiter(object):
         """\
         Kill unused/idle workers
         """
+        if not self.timeout:
+            return
         for (pid, worker) in self.WORKERS.items():
             try:
                 if time.time() - worker.tmp.last_update() <= self.timeout:
@@ -417,7 +431,7 @@ class Arbiter(object):
                     if not worker:
                         continue
                     worker.tmp.close()
-        except OSError, e:
+        except OSError as e:
             if e.errno == errno.ECHILD:
                 pass
 
@@ -430,7 +444,7 @@ class Arbiter(object):
             self.spawn_workers()
 
         workers = self.WORKERS.items()
-        workers.sort(key=lambda w: w[1].age)
+        workers = sorted(workers, key=lambda w: w[1].age)
         while len(workers) > self.num_workers:
             (pid, _) = workers.pop(0)
             self.kill_worker(pid, signal.SIGQUIT)
@@ -457,7 +471,7 @@ class Arbiter(object):
         except SystemExit:
             raise
         except:
-            self.log.debug("Exception in worker process:\n%s",
+            self.log.exception("Exception in worker process:\n%s",
                     traceback.format_exc())
             if not worker.booted:
                 sys.exit(self.WORKER_BOOT_ERROR)
@@ -498,7 +512,7 @@ class Arbiter(object):
          """
         try:
             os.kill(pid, sig)
-        except OSError, e:
+        except OSError as e:
             if e.errno == errno.ESRCH:
                 try:
                     worker = self.WORKERS.pop(pid)
