@@ -4,17 +4,16 @@
 # See the NOTICE for more information.
 
 import re
-import urlparse
-
-try:
-    from cStringIO import StringIO
-except ImportError:
-    from StringIO import StringIO
+import socket
+from errno import ENOTCONN
 
+from gunicorn.http.unreader import SocketUnreader
 from gunicorn.http.body import ChunkedReader, LengthReader, EOFReader, Body
 from gunicorn.http.errors import InvalidHeader, InvalidHeaderName, NoMoreData, \
 InvalidRequestLine, InvalidRequestMethod, InvalidHTTPVersion, \
 LimitRequestLine, LimitRequestHeaders
+from gunicorn.http.errors import InvalidProxyLine, ForbiddenProxyRequest
+from gunicorn.six import BytesIO, urlsplit, bytes_to_str
 
 MAX_REQUEST_LINE = 8190
 MAX_HEADERS = 32768
@@ -57,7 +56,7 @@ class Message(object):
         headers = []
 
         # Split lines on \r\n keeping the \r\n on each line
-        lines = [line + "\r\n" for line in data.split("\r\n")]
+        lines = [bytes_to_str(line) + "\r\n" for line in data.split(b"\r\n")]
 
         # Parse headers into key/value pairs paying attention
         # to continuation lines.
@@ -94,25 +93,27 @@ class Message(object):
 
     def set_body_reader(self):
         chunked = False
-        response_length = None
+        content_length = None
         for (name, value) in self.headers:
             if name == "CONTENT-LENGTH":
-                try:
-                    response_length = int(value)
-                except ValueError:
-                    response_length = None
+                content_length = value
             elif name == "TRANSFER-ENCODING":
                 chunked = value.lower() == "chunked"
             elif name == "SEC-WEBSOCKET-KEY1":
-                response_length = 8
-
-            if response_length is not None or chunked:
-                break
+                content_length = 8
 
         if chunked:
             self.body = Body(ChunkedReader(self, self.unreader))
-        elif response_length is not None:
-            self.body = Body(LengthReader(self.unreader, response_length))
+        elif content_length is not None:
+            try:
+                content_length = int(content_length)
+            except ValueError:
+                raise InvalidHeader("CONTENT-LENGTH", req=self)
+
+            if content_length < 0:
+                raise InvalidHeader("CONTENT-LENGTH", req=self)
+
+            self.body = Body(LengthReader(self.unreader, content_length))
         else:
             self.body = Body(EOFReader(self.unreader))
 
@@ -129,7 +130,7 @@ class Message(object):
 
 
 class Request(Message):
-    def __init__(self, cfg, unreader):
+    def __init__(self, cfg, unreader, req_number=1):
         self.methre = re.compile("[A-Z0-9$-_.]{3,20}")
         self.versre = re.compile("HTTP/(\d+).(\d+)")
 
@@ -144,6 +145,9 @@ class Request(Message):
         if (self.limit_request_line < 0
             or self.limit_request_line >= MAX_REQUEST_LINE):
             self.limit_request_line = MAX_REQUEST_LINE
+
+        self.req_number = req_number
+        self.proxy_protocol_info = None
         super(Request, self).__init__(cfg, unreader)
 
 
@@ -156,34 +160,31 @@ class Request(Message):
         buf.write(data)
 
     def parse(self, unreader):
-        buf = StringIO()
+        buf = BytesIO()
         self.get_data(unreader, buf, stop=True)
 
-        # Request line
-        data = buf.getvalue()
-        while True:
-            idx = data.find("\r\n")
-            if idx >= 0:
-                break
+        # get request line
+        line, rbuf = self.read_line(unreader, buf, self.limit_request_line)
 
-            if len(data) - 2 > self.limit_request_line > 0:
-                raise LimitRequestLine(len(data), self.limit_request_line)
+        # proxy protocol
+        if self.proxy_protocol(bytes_to_str(line)):
+            # get next request line
+            buf = BytesIO()
+            buf.write(rbuf)
+            line, rbuf = self.read_line(unreader, buf, self.limit_request_line)
 
-            self.get_data(unreader, buf)
-            data = buf.getvalue()
-
-        self.parse_request_line(data[:idx])
-        buf = StringIO()
-        buf.write(data[idx+2:]) # Skip \r\n
+        self.parse_request_line(bytes_to_str(line))
+        buf = BytesIO()
+        buf.write(rbuf)
 
         # Headers
         data = buf.getvalue()
-        idx = data.find("\r\n\r\n")
+        idx = data.find(b"\r\n\r\n")
 
-        done = data[:2] == "\r\n"
+        done = data[:2] == b"\r\n"
         while True:
-            idx = data.find("\r\n\r\n")
-            done = data[:2] == "\r\n"
+            idx = data.find(b"\r\n\r\n")
+            done = data[:2] == b"\r\n"
 
             if idx < 0 and not done:
                 self.get_data(unreader, buf)
@@ -195,14 +196,111 @@ class Request(Message):
 
         if done:
             self.unreader.unread(data[2:])
-            return ""
+            return b""
 
         self.headers = self.parse_headers(data[:idx])
 
         ret = data[idx+4:]
-        buf = StringIO()
+        buf = BytesIO()
         return ret
 
+    def read_line(self, unreader, buf, limit=0):
+        data = buf.getvalue()
+
+        while True:
+            idx = data.find(b"\r\n")
+            if idx >= 0:
+                # check if the request line is too large
+                if idx > limit > 0:
+                    raise LimitRequestLine(idx, limit)
+                break
+            self.get_data(unreader, buf)
+            data = buf.getvalue()
+
+            if len(data) - 2 > limit > 0:
+                raise LimitRequestLine(len(data), limit)
+
+        return (data[:idx], # request line,
+                data[idx + 2:]) #  residue in the buffer, skip \r\n
+
+    def proxy_protocol(self, line):
+        """\
+        Detect, check and parse proxy protocol.
+
+        :raises: ForbiddenProxyRequest, InvalidProxyLine.
+        :return: True for proxy protocol line else False
+        """
+        if not self.cfg.proxy_protocol:
+            return False
+
+        if self.req_number != 1:
+            return False
+
+        if not line.startswith("PROXY"):
+            return False
+
+        self.proxy_protocol_access_check()
+        self.parse_proxy_protocol(line)
+
+        return True
+
+    def proxy_protocol_access_check(self):
+        # check in allow list
+        if isinstance(self.unreader, SocketUnreader):
+            try:
+                remote_host = self.unreader.sock.getpeername()[0]
+            except socket.error as e:
+                if e.args[0] == ENOTCONN:
+                    raise ForbiddenProxyRequest("UNKNOW")
+                raise
+            if remote_host not in self.cfg.proxy_allow_ips:
+                raise ForbiddenProxyRequest(remote_host)
+
+    def parse_proxy_protocol(self, line):
+        bits = line.split()
+
+        if len(bits) != 6:
+            raise InvalidProxyLine(line)
+
+        # Extract data
+        proto = bits[1]
+        s_addr = bits[2]
+        d_addr = bits[3]
+
+        # Validation
+        if proto not in ["TCP4", "TCP6"]:
+            raise InvalidProxyLine("protocol '%s' not supported" % proto)
+        if proto == "TCP4":
+            try:
+                socket.inet_pton(socket.AF_INET, s_addr)
+                socket.inet_pton(socket.AF_INET, d_addr)
+            except socket.error:
+                raise InvalidProxyLine(line)
+        elif proto == "TCP6":
+            try:
+                socket.inet_pton(socket.AF_INET6, s_addr)
+                socket.inet_pton(socket.AF_INET6, d_addr)
+            except socket.error:
+                raise InvalidProxyLine(line)
+
+        try:
+            s_port = int(bits[4])
+            d_port = int(bits[5])
+        except ValueError:
+            raise InvalidProxyLine("invalid port %s" % line)
+
+        if not ((0 <= s_port <= 65535) and (0 <= d_port <= 65535)):
+            raise InvalidProxyLine("invalid port %s" % line)
+
+        # Set data
+        self.proxy_protocol_info = {
+            "proxy_protocol": proto,
+            "client_addr": s_addr,
+            "client_port": s_port,
+            "proxy_addr": d_addr,
+            "proxy_port": d_port
+        }
+
     def parse_request_line(self, line):
         bits = line.split(None, 2)
         if len(bits) != 3:
@@ -224,7 +322,7 @@ class Request(Message):
         else:
             self.uri = bits[1]
 
-        parts = urlparse.urlsplit(self.uri)
+        parts = urlsplit(self.uri)
         self.path = parts.path or ""
         self.query = parts.query or ""
         self.fragment = parts.fragment or ""
