@@ -1,14 +1,15 @@
 # -*- coding: utf-8 -
 #
-# This file is part of gunicorn released under the MIT license. 
+# This file is part of gunicorn released under the MIT license.
 # See the NOTICE for more information.
 
 import logging
 import os
 import re
 import sys
-from urllib import unquote
 
+from gunicorn.six import (unquote, string_types, binary_type, reraise,
+        text_type)
 from gunicorn import SERVER_SOFTWARE
 import gunicorn.util as util
 
@@ -39,10 +40,9 @@ class FileWrapper:
             return data
         raise IndexError
 
-def create(req, sock, client, server, cfg):
-    resp = Response(req, sock)
 
-    environ = {
+def default_environ(req, sock, cfg):
+    return {
         "wsgi.input": req.body,
         "wsgi.errors": sys.stderr,
         "wsgi.version": (1, 0),
@@ -55,29 +55,50 @@ def create(req, sock, client, server, cfg):
         "REQUEST_METHOD": req.method,
         "QUERY_STRING": req.query,
         "RAW_URI": req.uri,
-        "SERVER_PROTOCOL": "HTTP/%s" % ".".join(map(str, req.version)),
-        "CONTENT_TYPE": "",
-        "CONTENT_LENGTH": ""
+        "SERVER_PROTOCOL": "HTTP/%s" % ".".join([str(v) for v in req.version])
     }
-    
+
+def proxy_environ(req):
+    info = req.proxy_protocol_info
+
+    if not info:
+        return {}
+
+    return {
+        "PROXY_PROTOCOL": info["proxy_protocol"],
+        "REMOTE_ADDR": info["client_addr"],
+        "REMOTE_PORT":  str(info["client_port"]),
+        "PROXY_ADDR": info["proxy_addr"],
+        "PROXY_PORT": str(info["proxy_port"]),
+    }
+
+def create(req, sock, client, server, cfg):
+    resp = Response(req, sock)
+
+    environ = default_environ(req, sock, cfg)
+
     # authors should be aware that REMOTE_HOST and REMOTE_ADDR
     # may not qualify the remote addr:
     # http://www.ietf.org/rfc/rfc3875
-    client = client or "127.0.0.1"
-    forward = client
+    forward = client or "127.0.0.1"
     url_scheme = "http"
     script_name = os.environ.get("SCRIPT_NAME", "")
 
-    secure_headers = getattr(cfg, "secure_scheme_headers")
+    secure_headers = cfg.secure_scheme_headers
+    x_forwarded_for_header = cfg.x_forwarded_for_header
+    if '*' not in cfg.forwarded_allow_ips and client\
+            and client[0] not in cfg.forwarded_allow_ips:
+        x_forwarded_for_header = None
+        secure_headers = {}
 
     for hdr_name, hdr_value in req.headers:
         if hdr_name == "EXPECT":
             # handle expect
             if hdr_value.lower() == "100-continue":
                 sock.send("HTTP/1.1 100 Continue\r\n\r\n")
-        elif hdr_name == "X-FORWARDED-FOR":
+        elif x_forwarded_for_header and hdr_name == x_forwarded_for_header:
             forward = hdr_value
-        elif (hdr_name.upper() in secure_headers and
+        elif secure_headers and (hdr_name.upper() in secure_headers and
               hdr_value == secure_headers[hdr_name.upper()]):
             url_scheme = "https"
         elif hdr_name == "HOST":
@@ -90,13 +111,15 @@ def create(req, sock, client, server, cfg):
         elif hdr_name == "CONTENT-LENGTH":
             environ['CONTENT_LENGTH'] = hdr_value
             continue
-        
+
         key = 'HTTP_' + hdr_name.replace('-', '_')
+        if key in environ:
+            hdr_value = "%s,%s" % (environ[key], hdr_value)
         environ[key] = hdr_value
 
     environ['wsgi.url_scheme'] = url_scheme
-        
-    if isinstance(forward, basestring):
+
+    if isinstance(forward, string_types):
         # we only took the last one
         # http://en.wikipedia.org/wiki/X-Forwarded-For
         if forward.find(",") >= 0:
@@ -118,12 +141,12 @@ def create(req, sock, client, server, cfg):
 
         remote = (host, port)
     else:
-        remote = forward 
+        remote = forward
 
     environ['REMOTE_ADDR'] = remote[0]
     environ['REMOTE_PORT'] = str(remote[1])
 
-    if isinstance(server, basestring):
+    if isinstance(server, string_types):
         server =  server.split(":")
         if len(server) == 1:
             if url_scheme == "http":
@@ -133,7 +156,7 @@ def create(req, sock, client, server, cfg):
             else:
                 server.append('')
     environ['SERVER_NAME'] = server[0]
-    environ['SERVER_PORT'] = server[1]
+    environ['SERVER_PORT'] = str(server[1])
 
     path_info = req.path
     if script_name:
@@ -141,6 +164,8 @@ def create(req, sock, client, server, cfg):
     environ['PATH_INFO'] = unquote(path_info)
     environ['SCRIPT_NAME'] = script_name
 
+    environ.update(proxy_environ(req))
+
     return resp, environ
 
 class Response(object):
@@ -154,8 +179,9 @@ class Response(object):
         self.must_close = False
         self.headers = []
         self.headers_sent = False
-        self.clength = None
+        self.response_length = None
         self.sent = 0
+        self.upgrade = False
 
     def force_close(self):
         self.must_close = True
@@ -163,7 +189,7 @@ class Response(object):
     def should_close(self):
         if self.must_close or self.req.should_close():
             return True
-        if self.clength is not None or self.chunked:
+        if self.response_length is not None or self.chunked:
             return False
         return True
 
@@ -171,7 +197,7 @@ class Response(object):
         if exc_info:
             try:
                 if self.status and self.headers_sent:
-                    raise exc_info[0], exc_info[1], exc_info[2]
+                    reraise(exc_info[0], exc_info[1], exc_info[2])
             finally:
                 exc_info = None
         elif self.status is not None:
@@ -184,26 +210,31 @@ class Response(object):
 
     def process_headers(self, headers):
         for name, value in headers:
-            assert isinstance(name, basestring), "%r is not a string" % name
+            assert isinstance(name, string_types), "%r is not a string" % name
+
+            value = str(value).strip()
             lname = name.lower().strip()
             if lname == "content-length":
-                self.clength = int(value)
+                self.response_length = int(value)
             elif util.is_hoppish(name):
                 if lname == "connection":
                     # handle websocket
-                    if value.lower().strip() != "upgrade":
-                        continue
-                else:
-                    # ignore hopbyhop headers
-                    continue
-            self.headers.append((name.strip(), str(value).strip()))
+                    if value.lower().strip() == "upgrade":
+                        self.upgrade = True
+                elif lname == "upgrade":
+                    if value.lower().strip() == "websocket":
+                        self.headers.append((name.strip(), value))
+
+                # ignore hopbyhop headers
+                continue
+            self.headers.append((name.strip(), value))
 
 
     def is_chunked(self):
         # Only use chunked responses when the client is
         # speaking HTTP/1.1 or newer and there was
         # no Content-Length header set.
-        if self.clength is not None:
+        if self.response_length is not None:
             return False
         elif self.req.version <= (1,0):
             return False
@@ -214,9 +245,14 @@ class Response(object):
         return True
 
     def default_headers(self):
-        connection = "keep-alive"
-        if self.should_close():
+        # set the connection header
+        if self.upgrade:
+            connection = "upgrade"
+        elif self.should_close():
             connection = "close"
+        else:
+            connection = "keep-alive"
+
         headers = [
             "HTTP/%s.%s %s\r\n" % (self.req.version[0],
                 self.req.version[1], self.status),
@@ -232,22 +268,28 @@ class Response(object):
         if self.headers_sent:
             return
         tosend = self.default_headers()
-        tosend.extend(["%s: %s\r\n" % (n, v) for n, v in self.headers])
-        util.write(self.sock, "%s\r\n" % "".join(tosend))
+        tosend.extend(["%s: %s\r\n" % (k,v) for k, v in self.headers])
+
+        header_str = "%s\r\n" % "".join(tosend)
+        util.write(self.sock, util.to_bytestring(header_str))
         self.headers_sent = True
 
     def write(self, arg):
         self.send_headers()
-        assert isinstance(arg, basestring), "%r is not a string." % arg
+
+        if isinstance(arg, text_type):
+            arg = arg.encode('utf-8')
+
+        assert isinstance(arg, binary_type), "%r is not a byte." % arg
 
         arglen = len(arg)
         tosend = arglen
-        if self.clength is not None:
-            if self.sent >= self.clength:
-                # Never write more than self.clength bytes
+        if self.response_length is not None:
+            if self.sent >= self.response_length:
+                # Never write more than self.response_length bytes
                 return
 
-            tosend = min(self.clength - self.sent, tosend)
+            tosend = min(self.response_length - self.sent, tosend)
             if tosend < arglen:
                 arg = arg[:tosend]
 
@@ -287,8 +329,8 @@ class Response(object):
             fo_offset = respiter.filelike.tell()
             nbytes = max(os.fstat(fileno).st_size - fo_offset, 0)
 
-            if self.clength:
-                nbytes = min(nbytes, self.clength)
+            if self.response_length:
+                nbytes = min(nbytes, self.response_length)
 
             if nbytes == 0:
                 return
@@ -296,12 +338,13 @@ class Response(object):
             self.send_headers()
 
             if self.is_chunked():
-                self.sock.sendall("%X\r\n" % nbytes)
+                chunk_size = "%X\r\n" % nbytes
+                self.sock.sendall(chunk_size.encode('utf-8'))
 
             self.sendfile_all(fileno, self.sock.fileno(), fo_offset, nbytes)
 
             if self.is_chunked():
-                self.sock.sendall("\r\n")
+                self.sock.sendall(b"\r\n")
 
             os.lseek(fileno, fd_offset, os.SEEK_SET)
         else:
@@ -312,4 +355,4 @@ class Response(object):
         if not self.headers_sent:
             self.send_headers()
         if self.chunked:
-            util.write_chunk(self.sock, "")
+            util.write_chunk(self.sock, b"")
