@@ -13,6 +13,7 @@ import gunicorn.http as http
 import gunicorn.http.wsgi as wsgi
 import gunicorn.util as util
 import gunicorn.workers.base as base
+from gunicorn import six
 
 ALREADY_HANDLED = object()
 
@@ -31,32 +32,34 @@ class AsyncWorker(base.Worker):
             parser = http.RequestParser(self.cfg, client)
             try:
                 if not self.cfg.keepalive:
-                    req = parser.next()
+                    req = six.next(parser)
                     self.handle_request(req, client, addr)
                 else:
                     # keepalive loop
                     while True:
                         req = None
                         with self.timeout_ctx():
-                            req = parser.next()
+                            req = six.next(parser)
                         if not req:
                             break
                         self.handle_request(req, client, addr)
-            except http.errors.NoMoreData, e:
+            except http.errors.NoMoreData as e:
                 self.log.debug("Ignored premature client disconnection. %s", e)
-            except StopIteration, e:
+            except StopIteration as e:
                 self.log.debug("Closing connection. %s", e)
-            except Exception, e:
+            except socket.error:
+                raise # pass to next try-except level
+            except Exception as e:
                 self.handle_error(req, client, addr, e)
-        except socket.error, e:
-            if e[0] not in (errno.EPIPE, errno.ECONNRESET):
+        except socket.error as e:
+            if e.args[0] not in (errno.EPIPE, errno.ECONNRESET):
                 self.log.exception("Socket error processing request.")
             else:
-                if e[0] == errno.ECONNRESET:
+                if e.args[0] == errno.ECONNRESET:
                     self.log.debug("Ignoring connection reset")
                 else:
                     self.log.debug("Ignoring EPIPE")
-        except Exception, e:
+        except Exception as e:
             self.handle_error(req, client, addr, e)
         finally:
             util.close(client)
