@@ -8,7 +8,6 @@ import os
 import signal
 import sys
 import traceback
-import socket
 
 
 from gunicorn import util
@@ -18,13 +17,12 @@ InvalidRequestLine, InvalidRequestMethod, InvalidHTTPVersion, \
 LimitRequestLine, LimitRequestHeaders
 from gunicorn.http.errors import InvalidProxyLine, ForbiddenProxyRequest
 from gunicorn.http.wsgi import default_environ, Response
+from gunicorn.six import MAXSIZE
 
 class Worker(object):
 
-    SIGNALS = map(
-        lambda x: getattr(signal, "SIG%s" % x),
-        "HUP QUIT INT TERM USR1 USR2 WINCH CHLD".split()
-    )
+    SIGNALS = [getattr(signal, "SIG%s" % x) \
+            for x in "HUP QUIT INT TERM USR1 USR2 WINCH CHLD".split()]
 
     PIPE = []
 
@@ -43,7 +41,7 @@ class Worker(object):
         self.booted = False
 
         self.nr = 0
-        self.max_requests = cfg.max_requests or sys.maxint
+        self.max_requests = cfg.max_requests or MAXSIZE
         self.alive = True
         self.log = log
         self.debug = cfg.debug
@@ -87,8 +85,9 @@ class Worker(object):
 
         # For waking ourselves up
         self.PIPE = os.pipe()
-        map(util.set_non_blocking, self.PIPE)
-        map(util.close_on_exec, self.PIPE)
+        for p in self.PIPE:
+            util.set_non_blocking(p)
+            util.close_on_exec(p)
 
         # Prevent fd inherientence
         util.close_on_exec(self.socket)
@@ -105,7 +104,9 @@ class Worker(object):
         self.run()
 
     def init_signals(self):
-        map(lambda s: signal.signal(s, signal.SIG_DFL), self.SIGNALS)
+        # reset signaling
+        [signal.signal(s, signal.SIG_DFL) for s in self.SIGNALS]
+        # init new signaling
         signal.signal(signal.SIGQUIT, self.handle_quit)
         signal.signal(signal.SIGTERM, self.handle_exit)
         signal.signal(signal.SIGINT, self.handle_exit)
@@ -164,10 +165,6 @@ class Worker(object):
                                      error=str(exc),
                                     )
                           )
-        elif isinstance(exc, socket.timeout):
-            status_int = 408
-            reason = "Request Timeout"
-            mesg = "<p>The server timed out handling for the request</p>"
         else:
             self.log.exception("Error handling request")
 
